var ErpUtil = function(templateManager, General, ZhFilterPreferences, ZHPromise, ScreenService, restEngine, LibUtilities, MessageService, localStorageEngine) {

	this.getFieldValue = function(filedName, widgetName, fromFilter = false) {
		try {
			var container = templateManager.container;
			if (container) {
				let valuesWidget = container.getWidget(widgetName);
				if (fromFilter) {
					valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
				}

				if (valuesWidget) {
					var valueField = valuesWidget.currentRow[filedName];
					return valueField || null;
				}
				else return null;
			}
			else return null;
		}
		catch (error) {
			return null;
		}
	};

	// CRUD
	/**
	 * closeSwipe parametro que fecha ou nao o swipeApós o save
     * 
     * modeToReturn força o mode para o do parametro após o save, caso contrario volta para list para newRow e view para edit
     * 
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function(widget, closeSwipe = true, modeToReturn = undefined, afterValid = undefined, autoUpdateColumnsAfter = true) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
				widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function(success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.currentRow.__is_new = false;
					widget.currentRow.__createdLocal = false;

					if (modeToReturn)
						widget[modeToReturn]();
					else
						widget.list(closeSwipe);

					if (closeSwipe)
						ScreenService.goBack();

					widget.reload().then(function() {
						deferred.resolve(success);
					});
				}
				else {
					widget.computeDescriptionOrder();

					if (modeToReturn)
						widget[modeToReturn]();
					else
						widget.view();

					deferred.resolve(success);
				}

				if (autoUpdateColumnsAfter && widget.currentRow.__columns_to_return) {
					widget.currentRow.__columns_to_return.forEach(column => {
						widget.currentRow[column] = widget.currentRow.__columns_to_return[0][column];
					});
				}
			}, function(error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		}
		else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function(widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
						" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		}
		catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function(arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			}
			else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.parseStrToFloat = function(value) {
		if (!!value) {
			value = String(value);
			var numberOfDots = (value.match(/\./g) || []).length,
				hasVirgula = !!(value.match(/\,/g) || []).length;

			if (hasVirgula) {
				if (!!numberOfDots) {
					value = value.replace(/\./g, "");
				}

				value = value.replace(",", ".");
			}
		}
		else {
			value = 0.00;
		}

		return parseFloat(value);
	};

	this.rowEditedInline = function(row) {
		let boolEdited = false;

		$.each(row, function(index, value) {
			if (row.hasOwnProperty('changed_' + index)) {
				boolEdited = true;
			}
		});

		return boolEdited;
	};

	this.fixInlineEdit = function(row) {
		$.each(row, function(index, value) {
			if (row.hasOwnProperty('changed_' + index)) {
				row[index] = row[index + '_inline'] || row[index];
				row['changed_' + index] = row[index + '_inline'] || row[index];
			}
		});
		return row;
	};

	this.fixInlineEditDataSet = function(dataSet) {
		dataSet.forEach(row => {
			row = this.fixInlineEdit(row);
		});
		return dataSet;
	};

	this.saveDataSetCrud = function(dataSet, name) {
		return restEngine.post("est#/crud/" + name + "/save", {
			"requestType": "DataSet",
			"dataset": dataSet
		});
	};

	this.cancelInlineEdit = function(widget) {
		/* jshint ignore:start */

		// TODO criar função para desmarcar linhas editadas
		// GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);

		for (var index in widget.dataSource.data) {
			let row = widget.dataSource.data[index];
			Object.keys(row).forEach(function(key) {
				if (key.indexOf('changed_') > -1) {
					row[key.replace('changed_', '')] = row[key];
					delete row[key];
				}
			});
			delete row.changed;
		}

		widget.list();

		/* jshint ignore:end */
	};

	this.getFilterWidget = function(valuesWidget) {
		return templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
	};

	this.openBlankWindow = function(args) {
		args.e.preventDefault();
		ScreenService.openWindow("est#ger03302_blank_window");
	};

	this.saveCollectedDataGeneric = function(widget, description = undefined, fromFilter = false, ids = undefined) {
		let collectionIds;
		let valuesWidget = widget;
		if (fromFilter) {
			valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
		}

		if (valuesWidget) {
			if (valuesWidget.currentRow) {
				if (valuesWidget.currentRow.CDFILIMOVI)
					collectionIds = valuesWidget.currentRow.CDFILIMOVI;
				else if (valuesWidget.currentRow.CDFILIAL)
					collectionIds = valuesWidget.currentRow.CDFILIAL;
				else
					return null;
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}

		//widget instanceof Container
		let operationId = '';
		MessageService.getMessageByCode("UTLTELA").then(fraseComParametro => {
			operationId = ScreenService.i18n(fraseComParametro, { NMTELA: description });
			/*
			 * Lista de descriptions utilizadas para tentar padronizar chamadas:
			 * ContainerAfterinit
			 */
			/*
			 * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
			 * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
			 * @param $productId      => Código do Produto (opcional).
			 * @param $operatorCode   => Código do Operador (opcional).
			 * @param $operatorName   => Nome do Operador (opcional).
			 * @param $organizationId => Código do organização (opcional).
			 */
			LibUtilities.saveCollectedData(collectionIds, operationId);
		});
	};

	var getWidgetClientsByOperator = function() {
		return templateManager.container.getWidget('clientsByOperator');
	};

	// this.getLocalVar = function(key) {
	//     return localStorageEngine.getLocalVar(key);
	// };

	this.nextContainerAfterInit = function(container, route) {
		setTimeout(() => {

			var token = localStorage.getItem('LOGIN_TOKEN');
			var hash = localStorage.getItem('LOGIN_HASH') || '0';
			var languange = localStorage.getItem('LOGIN_LANGUAGE');
			var user = getLocalVar('NEXTSESSION_ENCRYPTED');
			var productId = window.projectId;

			var url = window.nextUrl + 'modules/erp-next/' + route + "?header=0&menu=0" +
				"&token=" + token +
				"&hash=" + hash +
				"&user=" + user +
				"&languange=" + languange +
				"&productId=" + productId +
				"&isZhAngular=1";

			// Clear OM session timeout to use only OM Next session timeout.
			//clearTimeout(LibUtilities.expiredSessionMethod);

			container.widgets[0].sourceUrl = url;
			container.widgets[0].reload();
		}, 1000);
	};
	
	this.nextAclContainerAfterInit = function(container, route) {
		setTimeout(() => {

			var token = localStorage.getItem('LOGIN_TOKEN');
			var hash = localStorage.getItem('LOGIN_HASH') || '0';
			var languange = localStorage.getItem('LOGIN_LANGUAGE');
			var user = getLocalVar('NEXTSESSION_ENCRYPTED');
			var productId = window.projectId;

			var url = window.nextUrl + 'modules/acl/' + route + "?header=0&menu=0" +
				"&token=" + token +
				"&hash=" + hash +
				"&user=" + user +
				"&languange=" + languange +
				"&productId=" + productId +
				"&isZhAngular=1";

			// Clear OM session timeout to use only OM Next session timeout.
			//clearTimeout(LibUtilities.expiredSessionMethod);

			container.widgets[0].sourceUrl = url;
			container.widgets[0].reload();
		}, 1000);
	};

	// this.nextContainerAfterInit = function (container, route) {
	// 	try{
	// 		if(typeof memoryStorage !== 'undefined') {
	//         localStorageEngine.getLocalVar = memoryStorage.getLocalVar;
	//         localStorageEngine.setLocalVar = memoryStorage.setLocalVar;
	//         setLocalVar = memoryStorage.setLocalVar;
	//         getLocalVar = memoryStorage.getLocalVar;
	//     }

	// 		getLocalVar('loginPromise').then(()=>{
	//      var token = localStorage.getItem('LOGIN_TOKEN');
	//      var hash = localStorage.getItem('LOGIN_HASH') || '0';
	//      var user      = localStorage.getItem('NEXTSESSION_ENCRYPTED');
	//      var languange = localStorage.getItem('LOGIN_LANGUAGE');
	//      var productId = window.projectId;

	//      var url = window.nextUrl + 'modules/est-next/' + route + "?header=0&menu=0" +
	//                                                   "&token=" + token +
	//                                                   "&hash=" + hash +
	//                                                   "&user=" + user +
	//                                                   "&languange=" + languange +
	//                                                   "&productId=" + productId +
	//                                                   "&isZhAngular=1";

	//      // Clear OM session timeout to use only OM Next session timeout.
	//      //clearTimeout(LibUtilities.expiredSessionMethod);

	//      container.widgets[0].sourceUrl = url;
	// 		});
	// 	}catch(error){
	// 		return null;
	// 	}
	// };

	/*
	 * This function validate all filter required fields and checks if all of 
	 * them have values before opening a new row.
	 *
	 * It Only Checks Fields which has validations on filterProperties
	 *
	 * @param widget => The widget which contains add action,
	 * @param openNewRow => boolean value, it checks if the function will open a new row or not,
	 */

	this.onValidateFilterRequirement = function(widget, openNewRow = false) {
		let checkedFields = [];
		const filterWidget = this.getFilterWidget(widget);
		const requiredFields = widget.fields.filter((field) => {
			if (field.hasOwnProperty('filterProperties')) {
				if (field.filterProperties.hasOwnProperty('validations')) {
					if (field.filterProperties.validations.hasOwnProperty('required')) {
						if (field.filterProperties.validations.required)
							return field.name;
					}
				}
			}
		});

		if (typeof widget.appliedFilter == 'undefined') {
			ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
			MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
			return true;
		}
		const { FILTER_ROW: filteredFields } = widget.appliedFilter;

		requiredFields.map((reqField) => {
			for (const filField in filteredFields) {
				if (reqField.name === filField && filteredFields[filField]) {
					checkedFields.push(filField);
					break;
				}
			}
		});

		if (requiredFields.length !== checkedFields.length) {
			ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
			MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
			return true;
		}
		else {
			if (openNewRow) widget.newRow();
			return false;
		}
		
		// if (!filterWidget.isValid()) {
		// 	ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
		// 	MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
		// 	return true;
		// }
		// else {
		// 	if (openNewRow) widget.newRow();
		// 	return false;
		// }
	};
	
	this.customFilter = function(args) {
	    let widgetTela = args.owner;
	    let filterWidget = templateManager.container.factoredWidgets.filter((widget) => widget._filteredWidgetId == widgetTela.id)[0];
	    let filterVisibleFields = filterWidget.fields.filter(fld=>fld.isVisible);
	    let fieldsToFilter = filterVisibleFields.filter(fld=>!fld.ignoreOnFilter);
	    let customFilter = {};
	    for(var campo in args.e.preference.FILTER_ROW){
	        if(fieldsToFilter.some(fld=> (fld.name == campo))) // jshint ignore:line
	            customFilter[campo] = args.e.preference.FILTER_ROW[campo];
	    }
	    args.e.preference.CUSTOM_FILTER_ROW = customFilter;
	};
	
};

Configuration(function(ContextRegister) {
	ContextRegister.register('ErpUtil', ErpUtil);
});
